AstNodeRef

Struct AstNodeRef 

pub struct AstNodeRef<T> { /* private fields */ }
Reference to an AST node.

This type acts as a reference to an AST node within a given module that remains stable regardless of whether the AST is garbage collected. As such, accessing a node through the AstNodeRef requires a reference to the current ParsedModuleRef for the module containing the node.

Usage in salsa tracked structs

It’s important that AstNodeRef fields in salsa tracked structs are tracked fields (attributed with #[tracked]). It prevents that the tracked struct gets a new ID every time the AST changes, which in turn, invalidates the result of any query that takes said tracked struct as a query argument or returns the tracked struct as part of its result.

For example, marking the AstNodeRef as tracked on Expression has the effect that salsa will consider the expression as “unchanged” for as long as it:

  • belongs to the same file
  • belongs to the same scope
  • has the same kind
  • was created in the same order

This means that changes to expressions in other scopes don’t invalidate the expression’s id, giving us some form of scope-stable identity for expressions. Only queries accessing the node field run on every AST change. All other queries only run when the expression’s identity changes.

Implementations

Source

impl<T> AstNodeRef<T>

pub fn index(&self) -> NodeIndex

Source

impl<T> AstNodeRef<T>
where T: HasNodeIndex + Ranged + PartialEq + Debug, for<'ast> AnyNodeRef<'ast>: From<&'ast T>, for<'ast> &'ast T: TryFrom<AnyRootNodeRef<'ast>>,

pub fn node<'ast>(&self, module_ref: &'ast ParsedModuleRef) -> &'ast T

Returns a reference to the wrapped node.

This method may panic or produce unspecified results if the provided module is from a different file, Python version, or Salsa revision than the module to which the node belongs.