Module definition

Module definition 

Structs

AnnotatedAssignmentDefinitionKind
AssignmentDefinitionKind
ComprehensionDefinitionKind
Note that the elements of a comprehension can be in different scopes. If the definition target of a comprehension is a name, it is in the comprehension’s scope. But if the target is an attribute or subscript, its definition is not in the comprehension’s scope; it is in the scope in which the root variable is bound. TODO: currently we don’t model this correctly and simply assume that it is in a scope outside the comprehension.
Definition
A definition of a place.
DefinitionNodeKey
Definitions
One or more Definitions.
DictKeyAssignmentKind
ExceptHandlerDefinitionKind
ForStmtDefinitionKind
ImportDefinitionKind
ImportFromDefinitionKind
ImportFromSubmoduleDefinitionKind
LambdaParameterDefinitionNodeKind
LoopHeaderDefinitionKind
Definition kind for a loop header entry.
MatchPatternDefinitionKind
NestedBindingsDefinitionKind
StarImportDefinitionKind
WithItemDefinitionKind

Enums

DefinitionCategory
DefinitionKind
The kind of a definition.
DefinitionPlace
The identity of the place defined by a Definition and whether it is re-exported.
DefinitionState
NestedBindingExecution
Describes when writes from a nested scope can affect its containing scope.
ParameterDefinitionNodeKind
TargetKind

Functions

docstring_from_body
Extract a docstring from a function, module, or class body.