pub enum DefinitionKind<'db> {
Show 24 variants
Import(ImportDefinitionKind),
ImportFrom(ImportFromDefinitionKind),
ImportFromSubmodule(ImportFromSubmoduleDefinitionKind),
StarImport(StarImportDefinitionKind),
Function(AstNodeRef<StmtFunctionDef>),
Class(AstNodeRef<StmtClassDef>),
TypeAlias(AstNodeRef<StmtTypeAlias>),
NamedExpression(AstNodeRef<ExprNamed>),
Assignment(AssignmentDefinitionKind<'db>),
AnnotatedAssignment(AnnotatedAssignmentDefinitionKind),
AugmentedAssignment(AstNodeRef<StmtAugAssign>),
DictKeyAssignment(DictKeyAssignmentKind<'db>),
For(ForStmtDefinitionKind<'db>),
Comprehension(ComprehensionDefinitionKind<'db>),
Parameter(ParameterDefinitionNodeKind),
LambdaParameter(LambdaParameterDefinitionNodeKind),
WithItem(WithItemDefinitionKind<'db>),
MatchPattern(MatchPatternDefinitionKind<'db>),
ExceptHandler(ExceptHandlerDefinitionKind),
TypeVar(AstNodeRef<TypeParamTypeVar>),
ParamSpec(AstNodeRef<TypeParamParamSpec>),
TypeVarTuple(AstNodeRef<TypeParamTypeVarTuple>),
LoopHeader(LoopHeaderDefinitionKind),
NestedBindings(Box<NestedBindingsDefinitionKind>),
}Expand description
The kind of a definition.
Usage in salsa tracked structs
DefinitionKind fields in salsa tracked structs should be tracked (attributed with #[tracked])
because the kind is a thin wrapper around AstNodeRef. See the AstNodeRef documentation
for an in-depth explanation of why this is necessary.
Variants
Import(ImportDefinitionKind)
ImportFrom(ImportFromDefinitionKind)
ImportFromSubmodule(ImportFromSubmoduleDefinitionKind)
StarImport(StarImportDefinitionKind)
Function(AstNodeRef<StmtFunctionDef>)
Class(AstNodeRef<StmtClassDef>)
TypeAlias(AstNodeRef<StmtTypeAlias>)
NamedExpression(AstNodeRef<ExprNamed>)
Assignment(AssignmentDefinitionKind<'db>)
AnnotatedAssignment(AnnotatedAssignmentDefinitionKind)
AugmentedAssignment(AstNodeRef<StmtAugAssign>)
DictKeyAssignment(DictKeyAssignmentKind<'db>)
For(ForStmtDefinitionKind<'db>)
Comprehension(ComprehensionDefinitionKind<'db>)
Parameter(ParameterDefinitionNodeKind)
LambdaParameter(LambdaParameterDefinitionNodeKind)
WithItem(WithItemDefinitionKind<'db>)
MatchPattern(MatchPatternDefinitionKind<'db>)
ExceptHandler(ExceptHandlerDefinitionKind)
TypeVar(AstNodeRef<TypeParamTypeVar>)
ParamSpec(AstNodeRef<TypeParamParamSpec>)
TypeVarTuple(AstNodeRef<TypeParamTypeVarTuple>)
LoopHeader(LoopHeaderDefinitionKind)
NestedBindings(Box<NestedBindingsDefinitionKind>)
Implementations
Sourceimpl<'db> DefinitionKind<'db>
impl<'db> DefinitionKind<'db>
pub const fn as_star_import(&self) -> Option<&StarImportDefinitionKind>
pub const fn as_class(&self) -> Option<&AstNodeRef<StmtClassDef>>
pub fn is_import(&self) -> bool
pub const fn is_unannotated_assignment(&self) -> bool
pub const fn is_function_def(&self) -> bool
pub const fn is_parameter_def(&self) -> bool
pub const fn is_loop_header(&self) -> bool
Sourcepub const fn is_user_visible(&self) -> bool
pub const fn is_user_visible(&self) -> bool
Returns true if this definition is user-visible (i.e., not an internal
synthetic definition like a loop header or nested bindings definition).
Sourcepub fn target_range(&self, module: &ParsedModuleRef) -> TextRange
pub fn target_range(&self, module: &ParsedModuleRef) -> TextRange
Returns the TextRange of the definition target.
A definition target would mainly be the node representing the place being defined i.e.,
ast::ExprName, ast::Identifier, ast::ExprAttribute or ast::ExprSubscript but could also be other nodes.
Sourcepub fn full_range(&self, module: &ParsedModuleRef) -> TextRange
pub fn full_range(&self, module: &ParsedModuleRef) -> TextRange
Returns the TextRange of the entire definition.
pub fn category( &self, in_stub: bool, module: &ParsedModuleRef, ) -> DefinitionCategory
Sourcepub fn value<'ast>(&self, module: &'ast ParsedModuleRef) -> Option<&'ast Expr>
pub fn value<'ast>(&self, module: &'ast ParsedModuleRef) -> Option<&'ast Expr>
Returns the value expression for assignment-based definitions.
Returns Some for Assignment and AnnotatedAssignment (if it has a value),
None for all other definition kinds.