pub fn docstring_from_body(body: &[Stmt]) -> Option<&ExprStringLiteral>Expand description
Extract a docstring from a function, module, or class body.
pub fn docstring_from_body(body: &[Stmt]) -> Option<&ExprStringLiteral>Extract a docstring from a function, module, or class body.