Definition

Struct Definition 

pub struct Definition<'db>(/* private fields */);
A definition of a place.

ID stability

The Definition’s ID is stable when the only field that change is its kind (AST node).

The Definition changes when the file, scope, or place change. This can be because a new scope gets inserted before the Definition or a new place is inserted before this Definition. However, the ID can be considered stable and it is okay to use Definition in cross-module` salsa queries or as a field on other salsa tracked structs.

Implementations

Source

impl Definition<'static>

pub fn ingredient(db: &dyn Database) -> &IngredientImpl<Self>

Source

impl<'db> Definition<'db>

pub fn new_internal<Db_>( db: &'db Db_, scope_id: ScopeId<'db>, place_info: DefinitionPlace, kind: DefinitionKind<'db>, ) -> Self
where Db_: ?Sized + Database,

pub fn kind<Db_>(self, db: &'db Db_) -> &'db DefinitionKind<'db>
where Db_: ?Sized + Database,

WARNING: Only access this field when doing type inference for the same file as where Definition is defined to avoid cross-file query dependencies.

pub fn scope_id<Db_>(self, db: &'db Db_) -> ScopeId<'db>
where Db_: ?Sized + Database,

The scope in which the definition occurs.

Storing the interned scope avoids retaining the file and file-local scope separately, at the cost of database lookups when either of those values is needed.

Source

impl<'_db> Definition<'_db>

pub fn default_debug_fmt(this: Self, f: &mut Formatter<'_>) -> Result
where for<'db> ScopeId<'db>: Debug, for<'db> DefinitionPlace: Debug, for<'db> DefinitionKind<'db>: Debug,

Default debug formatting for this struct (may be useful if you define your own Debug impl)

Source

impl<'db> Definition<'db>

pub fn scope(self, db: &'db dyn Db) -> ScopeId<'db>

pub fn file(self, db: &'db dyn Db) -> File

pub fn python_file(self, db: &'db dyn Db) -> PythonFile<'db>

pub fn program_file(self, db: &'db dyn Db) -> ProgramFile<'db>

pub fn program(self, db: &'db dyn Db) -> Program<'db>

pub fn file_scope(self, db: &'db dyn Db) -> FileScopeId

pub fn place(self, db: &'db dyn Db) -> ScopedPlaceId

pub fn is_reexported(self, db: &'db dyn Db) -> bool

pub fn full_range(self, db: &'db dyn Db, module: &ParsedModuleRef) -> FileRange

pub fn focus_range(self, db: &'db dyn Db, module: &ParsedModuleRef) -> FileRange

pub fn name(self, db: &'db dyn Db) -> Option<String>

Returns the name of the item being defined, if applicable.

pub fn docstring(self, db: &'db dyn Db) -> Option<String>

Extract a docstring from this definition, if applicable. This method returns a docstring for function, class, and attribute definitions. The docstring is extracted from the first statement in the body if it’s a string literal.