pub struct NestedBindingsDefinitionKind {
pub name: Name,
pub execution: NestedBindingExecution,
pub nested_declarations: SmallVec<[NestedDeclaration; 1]>,
}Fields
name: Nameexecution: NestedBindingExecutionnested_declarations: SmallVec<[NestedDeclaration; 1]>Implementations
Sourceimpl NestedBindingsDefinitionKind
impl NestedBindingsDefinitionKind
Sourcepub fn visible_binding_sources<'index, 'db>(
&'index self,
index: &'index SemanticIndex<'db>,
scope: FileScopeId,
) -> impl Iterator<Item = BindingWithConstraintsIterator<'index, 'db>> + 'index
pub fn visible_binding_sources<'index, 'db>( &'index self, index: &'index SemanticIndex<'db>, scope: FileScopeId, ) -> impl Iterator<Item = BindingWithConstraintsIterator<'index, 'db>> + 'index
Returns nested binding sources that can update the same variable as scope.
A synthetic binding can collect both global and nonlocal writes to one name:
x = 0
def outer():
x = 1
def change_global():
global x
x = 2
def change_nonlocal():
nonlocal x
x = 3Only change_nonlocal can update outer’s local x. Nested functions also cannot
capture a class-local variable, so class scopes do not see nonlocal writes to their
own bindings.