attribute_scopes

Function attribute_scopes 

pub fn attribute_scopes<'db>(
    db: &'db dyn Db,
    class_body_scope: ScopeId<'db>,
) -> impl Iterator<Item = FileScopeId> + 'db
Returns all attribute assignments as scope IDs for a specific class body scope.

Only call this when doing type inference on the same file as class_body_scope, otherwise it introduces a direct dependency on that file’s AST.