Module place

Module place 

Structs

ParentPlaceIter
PlaceTable
PlaceTableBuilder

Enums

PlaceExpr
An expression that can be the target of a Definition.
PlaceExprRef
Reference to a place expression, which can be a symbol or a member expression.
ScopedPlaceId
ID that uniquely identifies a place inside a Scope.