pub enum PythonPlatform {
All,
Identifier(String),
}Expand description
The target platform to assume when resolving types.
Variants
All
Do not make any assumptions about the target platform.
Identifier(String)
Assume a specific target platform like linux, darwin or win32.
We use a string (instead of individual enum variants), as the set of possible platforms may change over time. See https://docs.python.org/3/library/sys.html#sys.platform for some known platform identifiers.