pub struct PatternPredicate<'db>(/* private fields */);Implementations
Sourceimpl PatternPredicate<'static>
impl PatternPredicate<'static>
pub fn ingredient(db: &dyn Database) -> &IngredientImpl<Self>
Sourceimpl<'db> PatternPredicate<'db>
impl<'db> PatternPredicate<'db>
pub fn new<Db_>( db: &'db Db_, program_file: ProgramFile<'db>, file_scope: FileScopeId, subject: Expression<'db>, kind: PatternPredicateKind<'db>, guard: Option<Expression<'db>>, previous_predicate: Option<Box<PatternPredicate<'db>>>, ) -> Self
pub fn program_file<Db_>(self, db: &'db Db_) -> ProgramFile<'db>
pub fn file_scope<Db_>(self, db: &'db Db_) -> FileScopeId
pub fn subject<Db_>(self, db: &'db Db_) -> Expression<'db>
pub fn kind<Db_>(self, db: &'db Db_) -> &'db PatternPredicateKind<'db>
pub fn guard<Db_>(self, db: &'db Db_) -> Option<Expression<'db>>
Sourcepub fn previous_predicate<Db_>(
self,
db: &'db Db_,
) -> <Option<Box<PatternPredicate<'db>>> as SalsaAsDeref>::AsDeref<'db>
pub fn previous_predicate<Db_>( self, db: &'db Db_, ) -> <Option<Box<PatternPredicate<'db>>> as SalsaAsDeref>::AsDeref<'db>
A reference to the pattern of the previous match case
Sourceimpl<'_db> PatternPredicate<'_db>
impl<'_db> PatternPredicate<'_db>
Sourcepub fn default_debug_fmt(this: Self, f: &mut Formatter<'_>) -> Resultwhere
for<'db> ProgramFile<'db>: Debug,
for<'db> FileScopeId: Debug,
for<'db> Expression<'db>: Debug,
for<'db> PatternPredicateKind<'db>: Debug,
for<'db> Option<Expression<'db>>: Debug,
for<'db> Option<Box<PatternPredicate<'db>>>: Debug,
pub fn default_debug_fmt(this: Self, f: &mut Formatter<'_>) -> Resultwhere
for<'db> ProgramFile<'db>: Debug,
for<'db> FileScopeId: Debug,
for<'db> Expression<'db>: Debug,
for<'db> PatternPredicateKind<'db>: Debug,
for<'db> Option<Expression<'db>>: Debug,
for<'db> Option<Box<PatternPredicate<'db>>>: Debug,
Default debug formatting for this struct (may be useful if you define your own Debug impl)