pub struct StarImportPlaceholderPredicate<'db>(/* private fields */);Expand description
A “placeholder predicate” that is used to model the fact that the boundness of a (possible)
definition or declaration caused by a * import cannot be fully determined until type-
inference time. This is essentially the same as a standard reachability constraint, so we reuse
the Predicate infrastructure to model it.
To illustrate, say we have a module exporter.py like so:
if <condition>:
class A: ...and we have a module importer.py like so:
A = 1
from exporter import *Since we cannot know whether or not A in importer.py as a result of the from exporter import * statement,
but place a predicate on it to record the fact that we don’t yet know whether this definition
will be visible from all control-flow paths or not. Essentially, we model importer.py as
something similar to this:
A = 1
if <star_import_placeholder_predicate>:
from a import AAt type-check time, the placeholder predicate for the A definition is evaluated by attempting
to resolve the A symbol in exporter.py’s global namespace:
- If it resolves to a definitely bound symbol, then the predicate resolves to [
Truthiness::AlwaysTrue] - If it resolves to an unbound symbol, then the predicate resolves to [
Truthiness::AlwaysFalse] - If it resolves to a possibly bound symbol, then the predicate resolves to [
Truthiness::Ambiguous]
Implementations
Sourceimpl StarImportPlaceholderPredicate<'static>
impl StarImportPlaceholderPredicate<'static>
pub fn ingredient(db: &dyn Database) -> &IngredientImpl<Self>
Sourceimpl<'db> StarImportPlaceholderPredicate<'db>
impl<'db> StarImportPlaceholderPredicate<'db>
pub fn new<Db_>( db: &'db Db_, importing_file: ProgramFile<'db>, symbol_id: ScopedSymbolId, referenced_file: ProgramFile<'db>, ) -> Self
pub fn importing_file<Db_>(self, db: &'db Db_) -> ProgramFile<'db>
Sourcepub fn symbol_id<Db_>(self, db: &'db Db_) -> ScopedSymbolId
pub fn symbol_id<Db_>(self, db: &'db Db_) -> ScopedSymbolId
Each symbol imported by a * import has a separate predicate associated with it:
this field identifies which symbol that is.
Note that a [ScopedPlaceId] is only meaningful if you also know the scope
it is relative to. For this specific struct, however, there’s no need to store a
separate field to hold the ID of the scope. StarImportPredicates are only created
for valid *-import definitions, and valid *-import definitions can only ever
exist in the global scope; thus, we know that the symbol_id here will be relative
to the global scope of the importing file.
pub fn referenced_file<Db_>(self, db: &'db Db_) -> ProgramFile<'db>
Sourceimpl<'_db> StarImportPlaceholderPredicate<'_db>
impl<'_db> StarImportPlaceholderPredicate<'_db>
Sourcepub fn default_debug_fmt(this: Self, f: &mut Formatter<'_>) -> Result
pub fn default_debug_fmt(this: Self, f: &mut Formatter<'_>) -> Result
Default debug formatting for this struct (may be useful if you define your own Debug impl)