pub struct Program<'db>(/* private fields */);Implementations
Sourceimpl Program<'static>
impl Program<'static>
pub fn ingredient(zalsa: &Zalsa) -> &IngredientImpl<Self>
Sourceimpl<'db> Program<'db>
impl<'db> Program<'db>
pub fn new<Db_, T0: Lookup<PythonPlatform> + Hash, T1: Lookup<ResolverEnvironment<'db>> + Hash>(
db: &'db Db_,
python_platform: T0,
resolver_environment: T1,
) -> Selfwhere
Db_: ?Sized + Database,
PythonPlatform: HashEqLike<T0>,
ResolverEnvironment<'db>: HashEqLike<T1>,
pub fn python_platform<Db_>(self, db: &'db Db_) -> &'db PythonPlatform
pub fn resolver_environment<Db_>(self, db: &'db Db_) -> ResolverEnvironment<'db>
Sourceimpl Program<'_>
impl Program<'_>
Sourcepub fn default_debug_fmt(this: Self, f: &mut Formatter<'_>) -> Result
pub fn default_debug_fmt(this: Self, f: &mut Formatter<'_>) -> Result
Default debug formatting for this struct (may be useful if you define your own Debug impl)
Sourceimpl<'db> Program<'db>
impl<'db> Program<'db>
Sourcepub fn from_settings(db: &'db dyn Db, settings: &ProgramSettings) -> Self
pub fn from_settings(db: &'db dyn Db, settings: &ProgramSettings) -> Self
Creates a program from settings whose search roots have already been registered.
pub fn python_version(self, db: &'db dyn Db) -> PythonVersion
pub fn search_paths(self, db: &'db dyn Db) -> &'db SearchPaths
pub fn program_file(self, db: &'db dyn Db, file: File) -> ProgramFile<'db>
pub fn custom_stdlib_search_path( self, db: &'db dyn Db, ) -> Option<&'db SystemPath>