Module scope

Module scope 

Structs

FileScopeId
ID that uniquely identifies a scope inside of a module.
Scope
ScopeId
A cross-module identifier of a scope that can be used as a salsa query parameter.

Enums

NodeWithScopeKey
NodeWithScopeKind
Node that introduces a new scope.
NodeWithScopeRef
Reference to a node that introduces a new scope.
ScopeKind
ScopeVisibility