StatementInner

Struct StatementInner 

pub struct StatementInner<'db>(/* private fields */);
An independently type-inferable statement.

Module-local type

This type should not be used as part of any cross-module API because it holds a reference to the AST node. Range-offset changes then propagate through all usages, and deserialization requires reparsing the entire module.

E.g. don’t use this type in:

  • a return type of a cross-module query
  • a field of a type that is a return type of a cross-module query
  • an argument of a cross-module query

Implementations

Source

impl StatementInner<'static>

pub fn ingredient(db: &dyn Database) -> &IngredientImpl<Self>

Source

impl<'db> StatementInner<'db>

pub fn new<Db_>( db: &'db Db_, program_file: ProgramFile<'db>, file_scope: FileScopeId, node_ref: AstNodeRef<Stmt>, ) -> Self
where Db_: ?Sized + Database,

pub fn node_ref<Db_>(self, db: &'db Db_) -> &'db AstNodeRef<Stmt>
where Db_: ?Sized + Database,

The statement node.

pub fn program_file<Db_>(self, db: &'db Db_) -> ProgramFile<'db>
where Db_: ?Sized + Database,

The file in which the statement occurs.

pub fn file_scope<Db_>(self, db: &'db Db_) -> FileScopeId
where Db_: ?Sized + Database,

The scope in which the statement occurs.

Source

impl<'_db> StatementInner<'_db>

pub fn default_debug_fmt(this: Self, f: &mut Formatter<'_>) -> Result
where for<'db> ProgramFile<'db>: Debug, for<'db> FileScopeId: Debug, for<'db> AstNodeRef<Stmt>: Debug,

Default debug formatting for this struct (may be useful if you define your own Debug impl)

Source

impl<'db> StatementInner<'db>

pub fn file(self, db: &'db dyn Db) -> File

pub fn python_file(self, db: &'db dyn Db) -> PythonFile<'db>

pub fn scope(self, db: &'db dyn Db) -> ScopeId<'db>

pub fn program(self, db: &'db dyn Db) -> Program<'db>