pub struct LoopHeader { /* private fields */ }Expand description
All the bindings made in a loop, which are visible to the entire loop via “loop header definitions” (a.k.a. loop-back bindings)
Loop control flow analysis needs a way for uses early in the loop to see bindings that come later, reflecting the fact that a previous iteration of the loop might’ve already executed those bindings. For example:
x = "A"
while some_condition():
# The loop entry value (in the first iteration) and the following binding (in other
# iterations) are both visible at this use.
reveal_type(x) # revealed: Literal["A", "B"]
x = "B"As an important special case, these loop header definitions also combine with fixpoint
iteration to let us infer int for loop variables. For example:
i = 0 # revealed: Literal[0]
while i < 1_000_000:
i += 1 # revealed: intThe add-assign statement i += 1 is both a use and a binding. The use sees the i = 0 binding
at the top, and it also sees itself via the loop header definition of i. When we infer the
type of i in or after the loop, fixpoint iteration produces an ever-expanding union of
literals (Literal[0, 1, 2, ...]) until we eventually reach the threshold for widening to
int and stop iterating. (See should_widen and widen_literal_types.)
Loop header definitions are synthesized before walking the loop body, but the bindings in the
header are only known after walking all loop-back edges. The builder reserves a LoopHeaderId
before the walk, fills the corresponding header afterward, and publishes the completed headers
in the scope’s UseDefMap.
Implementations
Sourceimpl LoopHeader
impl LoopHeader
pub fn bindings_for_place( &self, place: ScopedPlaceId, ) -> impl Iterator<Item = LiveBinding> + '_