pub struct Symbol { /* private fields */ }Expand description
A symbol in a given scope.
Implementations
Sourceimpl Symbol
impl Symbol
pub fn name(&self) -> &Name
Sourcepub fn is_declared(&self) -> bool
pub fn is_declared(&self) -> bool
Is the symbol declared in its containing scope?
Sourcepub fn is_nonlocal(&self) -> bool
pub fn is_nonlocal(&self) -> bool
Is the symbol nonlocal its containing scope?
Sourcepub fn is_local(&self) -> bool
pub fn is_local(&self) -> bool
Is the symbol defined in this scope, vs referring to some enclosing scope?
There are three common cases where a name refers to an enclosing scope:
- explicit
globalvariables - explicit
nonlocalvariables - “free” variables, which are used in a scope where they’re neither bound nor declared
Note that even if is_local is false, that doesn’t necessarily mean there’s an enclosing
scope that resolves the reference. The symbol could be a built-in like print, or a name
error at runtime, or a global variable added dynamically with e.g. globals().
XXX: There’s a fourth case that we don’t (can’t) handle here. A variable that’s bound or declared (anywhere) in a class body, but used before it’s bound (at runtime), resolves (unbelievably) to the global scope. For example:
x = 42
def f():
x = 43
class Foo:
print(x) # 42 (never 43)
if secrets.randbelow(2):
x = 44
print(x) # 42 or 44In cases like this, the resolution isn’t known until runtime, and in fact it varies from
one use to the next. The semantic index alone can’t resolve this, and instead it’s a
special case in type inference (see infer_place_load).
pub const fn is_reassigned(&self) -> bool