pub struct Unpack<'db>(/* private fields */);Expand description
This ingredient represents a single unpacking.
This is required to make use of salsa to cache the complete unpacking of multiple variables involved. It allows us to:
- Avoid doing structural match multiple times for each definition
- Avoid highlighting the same error multiple times
Module-local type
This type should not be used as part of any cross-module API because it holds a reference to the AST node. Range-offset changes then propagate through all usages, and deserialization requires reparsing the entire module.
E.g. don’t use this type in:
- a return type of a cross-module query
- a field of a type that is a return type of a cross-module query
- an argument of a cross-module query
Implementations
Sourceimpl<'db> Unpack<'db>
impl<'db> Unpack<'db>
pub fn new<Db_>( db: &'db Db_, program_file: ProgramFile<'db>, value_file_scope: FileScopeId, target_file_scope: FileScopeId, _target: AstNodeRef<Expr>, value: UnpackValue<'db>, ) -> Self
pub fn program_file<Db_>(self, db: &'db Db_) -> ProgramFile<'db>
Sourceimpl<'_db> Unpack<'_db>
impl<'_db> Unpack<'_db>
Sourcepub fn default_debug_fmt(this: Self, f: &mut Formatter<'_>) -> Resultwhere
for<'db> ProgramFile<'db>: Debug,
for<'db, 'db> FileScopeId: Debug,
for<'db> AstNodeRef<Expr>: Debug,
for<'db> UnpackValue<'db>: Debug,
pub fn default_debug_fmt(this: Self, f: &mut Formatter<'_>) -> Resultwhere
for<'db> ProgramFile<'db>: Debug,
for<'db, 'db> FileScopeId: Debug,
for<'db> AstNodeRef<Expr>: Debug,
for<'db> UnpackValue<'db>: Debug,
Default debug formatting for this struct (may be useful if you define your own Debug impl)
Sourceimpl<'db> Unpack<'db>
impl<'db> Unpack<'db>
pub fn file(self, db: &'db dyn Db) -> File
pub fn python_file(self, db: &'db dyn Db) -> PythonFile<'db>
pub fn target<'ast>( self, db: &'db dyn Db, parsed: &'ast ParsedModuleRef, ) -> &'ast Expr
Sourcepub fn target_scope(self, db: &'db dyn Db) -> ScopeId<'db>
pub fn target_scope(self, db: &'db dyn Db) -> ScopeId<'db>
Returns the scope where the unpack target expression belongs to.
pub fn program(self, db: &'db dyn Db) -> Program<'db>
Sourcepub fn range(self, db: &'db dyn Db, module: &ParsedModuleRef) -> TextRange
pub fn range(self, db: &'db dyn Db, module: &ParsedModuleRef) -> TextRange
Returns the range of the unpack target expression.