Structs
- Annotated
Assignment Definition Kind - Assignment
Definition Kind - Comprehension
Definition Kind - Note that the elements of a comprehension can be in different scopes. If the definition target of a comprehension is a name, it is in the comprehension’s scope. But if the target is an attribute or subscript, its definition is not in the comprehension’s scope; it is in the scope in which the root variable is bound. TODO: currently we don’t model this correctly and simply assume that it is in a scope outside the comprehension.
- Definition
- A definition of a place.
- Definition
Node Key - Definitions
- One or more
Definitions. - Dict
KeyAssignment Kind - Except
Handler Definition Kind - ForStmt
Definition Kind - Import
Definition Kind - Import
From Definition Kind - Import
From Submodule Definition Kind - Lambda
Parameter Definition Node Kind - Loop
Header Definition Kind - Definition kind for a loop header entry.
- Match
Pattern Definition Kind - Nested
Bindings Definition Kind - Star
Import Definition Kind - With
Item Definition Kind
Enums
- Definition
Category - Definition
Kind - The kind of a definition.
- Definition
Place - The identity of the place defined by a
Definitionand whether it is re-exported. - Definition
State - Nested
Binding Execution - Describes when writes from a nested scope can affect its containing scope.
- Parameter
Definition Node Kind - Target
Kind
Functions
- docstring_
from_ body - Extract a docstring from a function, module, or class body.