pub struct Expression<'db>(/* private fields */);Expand description
An independently type-inferable expression.
Includes constraint expressions (e.g. if tests) and the RHS of an unpacking assignment.
Module-local type
This type should not be used as part of any cross-module API because it holds a reference to the AST node. Range-offset changes then propagate through all usages, and deserialization requires reparsing the entire module.
E.g. don’t use this type in:
- a return type of a cross-module query
- a field of a type that is a return type of a cross-module query
- an argument of a cross-module query
Implementations
Sourceimpl Expression<'static>
impl Expression<'static>
pub fn ingredient(db: &dyn Database) -> &IngredientImpl<Self>
Sourceimpl<'db> Expression<'db>
impl<'db> Expression<'db>
pub fn new<Db_>( db: &'db Db_, scope_id: ScopeId<'db>, node_ref: AstNodeRef<Expr>, assigned_to: Option<AstNodeRef<StmtAssign>>, kind: ExpressionKind, ) -> Self
Sourcepub fn node_ref<Db_>(self, db: &'db Db_) -> &'db AstNodeRef<Expr>
pub fn node_ref<Db_>(self, db: &'db Db_) -> &'db AstNodeRef<Expr>
The expression node.
Sourcepub fn assigned_to<Db_>(self, db: &'db Db_) -> Option<AstNodeRef<StmtAssign>>
pub fn assigned_to<Db_>(self, db: &'db Db_) -> Option<AstNodeRef<StmtAssign>>
An assignment statement, if this expression is immediately used as the rhs of that assignment.
(Note that this is the immediately containing assignment — if a complex expression is
assigned to some target, only the outermost expression node has this set. The inner
expressions are used to build up the assignment result, and are not “immediately assigned”
to the target, and so have None for this field.)
Sourceimpl<'_db> Expression<'_db>
impl<'_db> Expression<'_db>
Sourcepub fn default_debug_fmt(this: Self, f: &mut Formatter<'_>) -> Resultwhere
for<'db> ScopeId<'db>: Debug,
for<'db> AstNodeRef<Expr>: Debug,
for<'db> Option<AstNodeRef<StmtAssign>>: Debug,
for<'db> ExpressionKind: Debug,
pub fn default_debug_fmt(this: Self, f: &mut Formatter<'_>) -> Resultwhere
for<'db> ScopeId<'db>: Debug,
for<'db> AstNodeRef<Expr>: Debug,
for<'db> Option<AstNodeRef<StmtAssign>>: Debug,
for<'db> ExpressionKind: Debug,
Default debug formatting for this struct (may be useful if you define your own Debug impl)
Sourceimpl<'db> Expression<'db>
impl<'db> Expression<'db>
pub fn scope(self, db: &'db dyn Db) -> ScopeId<'db>
pub fn file(self, db: &'db dyn Db) -> File
pub fn python_file(self, db: &'db dyn Db) -> PythonFile<'db>
pub fn program_file(self, db: &'db dyn Db) -> ProgramFile<'db>
pub fn program(self, db: &'db dyn Db) -> Program<'db>