pub struct ScopedReachabilityConstraintId(/* private fields */);Expand description
A ternary formula that defines under what conditions a binding is visible. (A ternary formula
is just like a boolean formula, but with Ambiguous as a third potential result. See the
module documentation for more details.)
The primitive atoms of the formula are super::predicate::Predicates, which express some
property of the runtime state of the code that we are analyzing.
We assume that each atom has a stable value each time that the formula is evaluated. An atom
that resolves to Ambiguous might be true or false, and we can’t tell which — but within that
evaluation, we assume that the atom has the same unknown value each time it appears. That
allows us to perform simplifications like A ∨ !A → true and A ∧ !A → false.
That means that when you are constructing a formula, you might need to create distinct atoms
for a particular super::predicate::Predicate, if your formula needs to consider how a
particular runtime property might be different at different points in the execution of the
program.
reachability constraints are normalized, so equivalent constraints are guaranteed to have equal IDs.
Implementations
Sourceimpl ScopedReachabilityConstraintId
impl ScopedReachabilityConstraintId
Sourcepub const ALWAYS_TRUE: ScopedReachabilityConstraintId
pub const ALWAYS_TRUE: ScopedReachabilityConstraintId
A special ID that is used for an “always true” / “always visible” constraint.
Sourcepub const AMBIGUOUS: ScopedReachabilityConstraintId
pub const AMBIGUOUS: ScopedReachabilityConstraintId
A special ID that is used for an ambiguous constraint.
Sourcepub const ALWAYS_FALSE: ScopedReachabilityConstraintId
pub const ALWAYS_FALSE: ScopedReachabilityConstraintId
A special ID that is used for an “always false” / “never visible” constraint.